#import "PrivacyPane.h"

#include "nsCOMPtr.h"
#include "nsIServiceManagerUtils.h"
#include "nsIPref.h"
#include "nsCCookieManager.h"
#include "nsICookie.h"
#include "nsIPermissionManager.h"
#include "nsISimpleEnumerator.h"
#include "nsIPermission.h"
#include "nsISupportsArray.h"
#include "nsString.h"


// prefs for keychain password autofill
static const char* const gUseKeychainPref = "chimera.store_passwords_with_keychain";
static const char* const gAutoFillEnabledPref = "chimera.keychain_passwords_autofill";

const int kPromptForCookiesTag = 99;
const int kEnableAllCookies = 0;
const int kEnableCookiesFromOriginatingServerOnly = 1;
const int kDisableAllCookies = 2;

@implementation OrgMozillaChimeraPreferencePrivacy

- (void) dealloc
{
  NS_IF_RELEASE(mPermissionManager);
  NS_IF_RELEASE(mCookieManager);
  [super dealloc];
}

- (void)mainViewDidLoad
{
  if ( !mPrefService )
    return;
    
  // Hookup cookie prefs. The "ask about" and "cookie sites" buttons need to
  // be disabled when cookies aren't enabled because they aren't applicable.
  PRInt32 acceptCookies = kEnableAllCookies;
  mPrefService->GetIntPref("network.cookie.cookieBehavior", &acceptCookies);
  NSLog(@"%d default cookie behaviour", acceptCookies);
  [mCookieBehaviour selectCellWithTag: acceptCookies];
  [self mapCookiePrefToGUI: acceptCookies];
  /* Obsolete
  BOOL cookiesEnabled = [self mapCookiePrefToCheckbox:acceptCookies];
  [mCookiesEnabled setState:cookiesEnabled];
  if ( !cookiesEnabled ) {
    [mAskAboutCookies setEnabled:NO];
    [mEditSitesButton setEnabled:NO];
    [mEditSitesText setTextColor:[NSColor lightGrayColor]];
  }
  */
  PRBool warnAboutCookies = PR_TRUE;
  mPrefService->GetBoolPref("network.cookie.warnAboutCookies", &warnAboutCookies);
  if ( warnAboutCookies )
    [mAskAboutCookies setState:YES];
  
  // store permission manager service and cache the enumerator.
  nsCOMPtr<nsIPermissionManager> pm ( do_GetService(NS_PERMISSIONMANAGER_CONTRACTID) );
  mPermissionManager = pm.get();
  NS_IF_ADDREF(mPermissionManager);

  // store cookie manager service
  nsCOMPtr<nsICookieManager> cm ( do_GetService(NS_COOKIEMANAGER_CONTRACTID) );
  mCookieManager = cm.get();
  NS_IF_ADDREF(mCookieManager);
  
  // Keychain checkboxes	
  PRBool storePasswords = PR_TRUE;
  mPrefService->GetBoolPref(gUseKeychainPref, &storePasswords);
  [mStorePasswords setState:(storePasswords ? NSOnState : NSOffState)];
  [mAutoFillPasswords setEnabled:storePasswords ? YES : NO];

  PRBool autoFillPasswords = PR_TRUE;
  mPrefService->GetBoolPref(gAutoFillEnabledPref, &autoFillPasswords);
  [mAutoFillPasswords setState:(autoFillPasswords ? NSOnState : NSOffState)];
}

-(IBAction) editCookieSites:(id)aSender
{
  // build parallel permission list for speed with a lot of blocked sites
  nsCOMPtr<nsISimpleEnumerator> permEnum;
  if ( mPermissionManager ) 
    mPermissionManager->GetEnumerator(getter_AddRefs(permEnum));

  NS_NewISupportsArray(&mCachedPermissions);     // ADDREFs
  if ( mCachedPermissions && permEnum ) {
    PRBool hasMoreElements = PR_FALSE;
    permEnum->HasMoreElements(&hasMoreElements);
    while ( hasMoreElements ) {
      nsCOMPtr<nsISupports> curr;
      permEnum->GetNext(getter_AddRefs(curr));
      nsCOMPtr<nsIPermission> currPerm(do_QueryInterface(curr));
      if ( currPerm ) {
        nsCAutoString type;
        currPerm->GetType(type);
        if ( type.Equals(NS_LITERAL_CSTRING("cookie")) )
          mCachedPermissions->AppendElement(curr);
      }
      permEnum->HasMoreElements(&hasMoreElements);
    }
  }

  // build parallel cookie list
  nsCOMPtr<nsISimpleEnumerator> cookieEnum;
	if ( mCookieManager )
		mCookieManager->GetEnumerator(getter_AddRefs(cookieEnum));

  NS_NewISupportsArray(&mCachedCookies);
	if ( mCachedCookies && cookieEnum ) {
		PRBool hasMoreElements = PR_FALSE;
		cookieEnum->HasMoreElements(&hasMoreElements);
		while ( hasMoreElements )
		{
      nsCOMPtr<nsISupports> curr;
			cookieEnum->GetNext(getter_AddRefs(curr));
			nsCOMPtr<nsICookie> currCookie(do_QueryInterface(curr));
      if ( currCookie )
        mCachedCookies->AppendElement(currCookie);
			//
			cookieEnum->HasMoreElements(&hasMoreElements);
		}
	}
  
	// bring up sheet
	[NSApp beginSheet:mCookieSitePanel
        modalForWindow:[mAskAboutCookies window]   // any old window accessor
        modalDelegate:self
        didEndSelector:@selector(editCookieSitesSheetDidEnd:returnCode:contextInfo:)
        contextInfo:NULL];
        
  // ensure a row is selected (cocoa doesn't do this for us, but will keep
  // us from unselecting a row once one is set; go figure).
  [mPermissionsTable selectRow:0 byExtendingSelection:NO];
	[mCookiesTable selectRow: 0 byExtendingSelection: NO];
  
  // we shouldn't need to do this, but the scrollbar won't enable unless we
  // force the table to reload its data. Oddly it gets the number of rows correct,
  // it just forgets to tell the scrollbar. *shrug*
  [mPermissionsTable reloadData];
  [mCookiesTable reloadData];
}

-(IBAction) editCookieSitesDone:(id)aSender
{
  // save stuff
  [mCookieSitePanel orderOut:self];
  [NSApp endSheet:mCookieSitePanel];
  
  NS_IF_RELEASE(mCachedPermissions);
  NS_IF_RELEASE(mCachedCookies);
}

- (void)editCookieSitesSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
}

-(IBAction) removeCookiePermissions:(id)aSender
{
  if ( mCachedPermissions && mPermissionManager ) {
    // reverse through selected indexes to 'solve' shifting indexes when deleting objects in array
    NSArray *rows = [[mPermissionsTable selectedRowEnumerator] allObjects];
    NSEnumerator *e = [rows reverseObjectEnumerator];
    NSNumber *index;
    while (index = [e nextObject])
    {
      int row = [index intValue];
      nsCOMPtr<nsISupports> rowItem = dont_AddRef(mCachedPermissions->ElementAt(row));
      nsCOMPtr<nsIPermission> perm ( do_QueryInterface(rowItem) );
      if ( perm ) {
        nsCAutoString host;
        perm->GetHost(host);
        mPermissionManager->Remove(host, "cookie");
        mCachedPermissions->RemoveElementAt(row);
      }
    }
  }

  [mPermissionsTable deselectAll: self];	// don't want any traces of previous selection
  [mPermissionsTable reloadData];
}

-(IBAction) removeAllCookiePermissions: (id)aSender
{
  if ( mPermissionManager ) {
    // remove all permissions from permission manager
    mPermissionManager->RemoveAll();
    // create new permissions cache
    NS_IF_RELEASE(mCachedPermissions);
    NS_NewISupportsArray(&mCachedPermissions);
  }

  [mPermissionsTable reloadData];
}

-(IBAction) removeCookies: (id)aSender
{
  if ( mCachedCookies && mCookieManager ) {
    // reverse through selected indexes to 'solve' shifting indexes when deleting objects in array
    NSArray *rows = [[mCookiesTable selectedRowEnumerator] allObjects];
    NSEnumerator *e = [rows reverseObjectEnumerator];
    NSNumber *index;
    while (index = [e nextObject])
    {
      // get cookie from cache
      int row = [index intValue];
      nsCOMPtr<nsISupports> rowItem = dont_AddRef(mCachedCookies->ElementAt(row));
      nsCOMPtr<nsICookie> cookie ( do_QueryInterface(rowItem) );
      if (cookie) {
        // remove cookie from cookie manager
        nsCAutoString host, name, path;
        cookie->GetHost(host);
        cookie->GetName(name);
        cookie->GetPath(path);
        mCookieManager->Remove(host, name, path, PR_FALSE);	// don't block permanently
        // remove cookie from cache
        mCachedCookies->RemoveElementAt(row);
      }
    }
  }

  [mCookiesTable deselectAll: self];	// don't want any traces of previous selection
	[mCookiesTable reloadData];
}

-(IBAction) removeAllCookies: (id)aSender
{
  if ( mCookieManager ) {
    // remove all cookies from cookie manager
    mCookieManager->RemoveAll();
    // create new cookie cache
    NS_IF_RELEASE(mCachedCookies);
    NS_NewISupportsArray(&mCachedCookies);
  }

  [mCookiesTable reloadData];
}

-(void) mapCookiePrefToGUI: (int)pref
{
  [mCookieBehaviour selectCellWithTag: pref];
  if (pref == 0 || pref == 1) {
    [mAskAboutCookies setEnabled: YES];
    [mEditSitesButton setEnabled: YES];
  } else {
    [mAskAboutCookies setEnabled: NO];
    [mEditSitesButton setEnabled: NO];
  }
}

//
// NSTableDataSource protocol methods
//

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
  PRUint32 numRows = 0;

  if (aTableView == mPermissionsTable) {
    if ( mCachedPermissions )
      mCachedPermissions->Count(&numRows);
  } else if (aTableView == mCookiesTable) {
    if ( mCachedCookies )
      mCachedCookies->Count(&numRows);
  }

  return (int) numRows;
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
  NSString* retVal = nil;
    
  if (aTableView == mPermissionsTable) {
    if ( mCachedPermissions ) {
      nsCOMPtr<nsISupports> rowItem = dont_AddRef(mCachedPermissions->ElementAt(rowIndex));
      nsCOMPtr<nsIPermission> perm ( do_QueryInterface(rowItem) );
      if ( perm ) {
        if ( [[aTableColumn identifier] isEqualToString:@"Website"] ) {
          // website url column
          nsCAutoString host;
          perm->GetHost(host);
          retVal = [NSString stringWithCString:host.get()];
        } else {
          // allow/deny column
          PRUint32 capability = PR_FALSE;
          perm->GetCapability(&capability);
          if ( capability == nsIPermissionManager::ALLOW_ACTION)
            retVal = [self getLocalizedString:@"Allow"];
          else
            retVal = [self getLocalizedString:@"Deny"];
        }
      }
    }
  } else if (aTableView == mCookiesTable) {
    if ( mCachedCookies ) {
      nsCOMPtr<nsISupports> rowItem = dont_AddRef(mCachedCookies->ElementAt(rowIndex));
      nsCOMPtr<nsICookie> cookie ( do_QueryInterface(rowItem) );
      if ( cookie ) {
        nsCAutoString cookieVal;
        //
        if ( [[aTableColumn identifier] isEqualToString: @"Website"] ) {
          cookie->GetHost(cookieVal);
        } else if ( [[aTableColumn identifier] isEqualToString: @"Name"] ) {
          cookie->GetName(cookieVal);
        } else if ( [[aTableColumn identifier] isEqualToString: @"Path"] ) {
          cookie->GetPath(cookieVal);
        } else if ( [[aTableColumn identifier] isEqualToString: @"Secure"] ) {
          PRBool secure = PR_FALSE;
          cookie->GetIsSecure(&secure);
          if (secure)
            cookieVal = "yes";
          else
            cookieVal = "no";
        } else if ( [[aTableColumn identifier] isEqualToString: @"Expires"] ) {
          PRUint64 expires = 0;
          cookie->GetExpires(&expires);
          NSDate *date = [NSDate dateWithTimeIntervalSince1970: (NSTimeInterval)expires];
          return date;	// special case return!
        } else if ( [[aTableColumn identifier] isEqualToString: @"Value"] ) {
          cookie->GetValue(cookieVal);
        }
        //
        retVal = [NSString stringWithCString: cookieVal.get()];
      }
    }
  }
    
  return retVal;
}

-(IBAction) clickCookieBehaviour: (id)aSender
{
  int row = [mCookieBehaviour selectedRow];
  [self setPref: "network.cookie.cookieBehavior" toInt: row];
  [self mapCookiePrefToGUI: row];
}

-(IBAction) clickAskAboutCookies:(id)sender
{
  [self setPref:"network.cookie.warnAboutCookies" toBoolean:[sender state]];
}


//
// clickStorePasswords
//
-(IBAction) clickStorePasswords:(id)sender
{
  if ( !mPrefService )
    return;
  if([mStorePasswords state] == NSOnState)
  {
      mPrefService->SetBoolPref("chimera.store_passwords_with_keychain", PR_TRUE);
      [mAutoFillPasswords setEnabled:YES];
  }
  else
  {
      mPrefService->SetBoolPref("chimera.store_passwords_with_keychain", PR_FALSE);
      [mAutoFillPasswords setEnabled:NO];
  }        
}

//
// clickAutoFillPasswords
//
// Set pref if autofill is enabled
//
-(IBAction) clickAutoFillPasswords:(id)sender
{
  if ( !mPrefService )
    return;
  mPrefService->SetBoolPref("chimera.keychain_passwords_autofill",
                            [mAutoFillPasswords state] == NSOnState ? PR_TRUE : PR_FALSE);
}

-(IBAction) launchKeychainAccess:(id)sender
{
  FSRef fsRef;
  CFURLRef urlRef;
  OSErr err = ::LSGetApplicationForInfo('APPL', 'kcmr', NULL, kLSRolesAll, &fsRef, &urlRef);
  if ( !err ) {
    CFStringRef fileSystemURL = ::CFURLCopyFileSystemPath(urlRef, kCFURLPOSIXPathStyle);
    [[NSWorkspace sharedWorkspace] launchApplication:(NSString*)fileSystemURL];
    CFRelease(fileSystemURL);
  }
}

@end
